==> SPVM-0.0221.tar.gz: SPVM-0.0221/spvm_dynamic_array.c <==
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "spvm_dynamic_array.h"
#include "spvm_util_allocator.h"
#include "spvm_compiler.h"

SPVM_DYNAMIC_ARRAY* SPVM_DYNAMIC_ARRAY_new(int32_t capacity) {
  
  assert(capacity >= 0);
  
  SPVM_DYNAMIC_ARRAY* array = SPVM_UTIL_ALLOCATOR_safe_malloc_i32(1, sizeof(SPVM_DYNAMIC_ARRAY));
  array->length = 0;
  
  if (capacity == 0) {
    array->capacity = 128;
  }
  else {
    array->capacity = capacity;
  }
  
  void** values = SPVM_UTIL_ALLOCATOR_safe_malloc_i32(array->capacity, sizeof(void*));
  
  array->values = values;
  
  return array;
}

void SPVM_DYNAMIC_ARRAY_maybe_extend(SPVM_DYNAMIC_ARRAY* array) {
  
  assert(array);
  
  int32_t length = array->length;
  int32_t capacity = array->capacity;
  
  if (length >= capacity) {
    int32_t new_capacity = capacity * 2;
    array->values = SPVM_UTIL_ALLOCATOR_safe_realloc_i32(array->values, new_capacity, sizeof(void*));
    array->capacity = new_capacity;
  }
}

void SPVM_DYNAMIC_ARRAY_free(SPVM_DYNAMIC_ARRAY* array) {
  
  free(array->values);
  free(array);
}

void SPVM_DYNAMIC_ARRAY_push(SPVM_DYNAMIC_ARRAY* array, void* value) {
  
  SPVM_DYNAMIC_ARRAY_maybe_extend(array);
  
  int32_t length = array->length;
  
  *(void**)&array->values[length] = value;
  array->length++;
}

void* SPVM_DYNAMIC_ARRAY_fetch(SPVM_DYNAMIC_ARRAY* array, int32_t index) {
  
  assert(array);
  assert(index >= 0);
  assert(index < array->length);
  
  return *(void**)&array->values[index];
}

void SPVM_DYNAMIC_ARRAY_store(SPVM_DYNAMIC_ARRAY* array, int32_t index, void* value) {
  
  assert(array);
  assert(index >= 0);
  assert(index < array->length);
  
  *(void**)&array->values[index] = value;
}

void* SPVM_DYNAMIC_ARRAY_pop(SPVM_DYNAMIC_ARRAY* array) {
  
  assert(array->length >= 0);
  
  if (array->length == 0) {
    return NULL;
  }
  else {
    array->length--;
    return *(void**)&array->values[array->length];
  }
}

==> SPVM-0.0226.tar.gz: SPVM-0.0226/spvm_dynamic_array.c <==
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "spvm_dynamic_array.h"
#include "spvm_util_allocator.h"
#include "spvm_compiler.h"

SPVM_DYNAMIC_ARRAY* SPVM_DYNAMIC_ARRAY_new(int32_t capacity) {
  
  assert(capacity >= 0);
  
  SPVM_DYNAMIC_ARRAY* array = SPVM_UTIL_ALLOCATOR_safe_malloc_i32(1, sizeof(SPVM_DYNAMIC_ARRAY));
  array->length = 0;
  
  if (capacity == 0) {
    array->capacity = 128;
  }
  else {
    array->capacity = capacity;
  }
  
  void** values = SPVM_UTIL_ALLOCATOR_safe_malloc_i32(array->capacity, sizeof(void*));
  
  array->values = values;
  
  return array;
}

void SPVM_DYNAMIC_ARRAY_maybe_extend(SPVM_DYNAMIC_ARRAY* array) {
  
  assert(array);
  
  int32_t length = array->length;
  int32_t capacity = array->capacity;
  
  if (length >= capacity) {
    int32_t new_capacity = capacity * 2;
    
    void** new_values = SPVM_UTIL_ALLOCATOR_safe_malloc_i32_zero(new_capacity, sizeof(void*));
    memcpy(new_values, array->values, capacity * sizeof(void*));
    free(array->values);
    array->values = new_values;

    array->capacity = new_capacity;
  }
}

void SPVM_DYNAMIC_ARRAY_free(SPVM_DYNAMIC_ARRAY* array) {
  
  free(array->values);
  free(array);
}

void SPVM_DYNAMIC_ARRAY_push(SPVM_DYNAMIC_ARRAY* array, void* value) {
  
  SPVM_DYNAMIC_ARRAY_maybe_extend(array);
  
  int32_t length = array->length;
  
  *(void**)&array->values[length] = value;
  array->length++;
}

void* SPVM_DYNAMIC_ARRAY_fetch(SPVM_DYNAMIC_ARRAY* array, int32_t index) {
  
  assert(array);
  assert(index >= 0);
  assert(index < array->length);
  
  return *(void**)&array->values[index];
}

void SPVM_DYNAMIC_ARRAY_store(SPVM_DYNAMIC_ARRAY* array, int32_t index, void* value) {
  
  assert(array);
  assert(index >= 0);
  assert(index < array->length);
  
  *(void**)&array->values[index] = value;
}

void* SPVM_DYNAMIC_ARRAY_pop(SPVM_DYNAMIC_ARRAY* array) {
  
  assert(array->length >= 0);
  
  if (array->length == 0) {
    return NULL;
  }
  else {
    array->length--;
    return *(void**)&array->values[array->length];
  }
}
